@@ -30,6 +30,8 @@
 #include <QPushButton>
 #include <QLineEdit>
 
+#include <utility>
+
 #include "collectionlist.h"
 #include "searchwidget.h"
 
@@ -107,14 +109,10 @@ AdvancedSearchDialog::AdvancedSearchDialog(const QString &defaultName,
         m_matchAnyButton->setChecked(true);
     }
     else {
-        PlaylistSearch::ComponentList components = defaultSearch.components();
-
-        for(PlaylistSearch::ComponentList::ConstIterator it = components.constBegin();
-            it != components.constEnd();
-            ++it)
-        {
+        const auto components = defaultSearch.components();
+        for(const auto &component : components) {
             SearchLine *s = new SearchLine(this);
-            s->setSearchComponent(*it);
+            s->setSearchComponent(component);
             m_searchLines.append(s);
             m_criteriaLayout->insertWidget(m_criteriaLayout->count() - 1, s);
         }
@@ -139,7 +137,7 @@ void AdvancedSearchDialog::accept()
 
     m_search->addPlaylist(CollectionList::instance());
 
-    for(const auto &searchLine : qAsConst(m_searchLines))
+    for(const auto &searchLine : std::as_const(m_searchLines))
         m_search->addComponent(searchLine->searchComponent());
 
     PlaylistSearch::SearchMode m = PlaylistSearch::SearchMode(!m_matchAnyButton->isChecked());
