@@ -37,6 +37,8 @@
 #include <QTimer>
 #include <QWriteLocker>
 
+#include <utility>
+
 #include "playlistcollection.h"
 #include "stringshare.h"
 #include "cache.h"
@@ -46,6 +48,7 @@
 #include "juk_debug.h"
 
 using ActionCollection::action;
+using std::as_const;
 
 ////////////////////////////////////////////////////////////////////////////////
 // static methods
@@ -103,11 +106,13 @@ void CollectionList::loadNextBatchCachedItems()
         }
     }
 
+    // Whatever gets done here cannot be done as a direct call since we have
+    // the items dict locked
     if(!done) {
         QTimer::singleShot(0, this, &CollectionList::loadNextBatchCachedItems);
     }
     else {
-        completedLoadingCachedItems();
+        QTimer::singleShot(0, this, &CollectionList::completedLoadingCachedItems);
     }
 }
 
@@ -173,7 +178,7 @@ CollectionListItem *CollectionList::createItem(const FileHandle &file, QTreeWidg
 
 void CollectionList::clearItems(const PlaylistItemList &items)
 {
-    foreach(PlaylistItem *item, items) {
+    for(PlaylistItem *item : items) {
         delete item;
     }
 
@@ -193,16 +198,18 @@ void CollectionList::setupTreeViewEntries(ViewMode *viewMode) const
     columnList << PlaylistItem::GenreColumn;
     columnList << PlaylistItem::AlbumColumn;
 
-    foreach(int column, columnList)
+    for(int column : as_const(columnList)) {
         treeViewMode->addItems(m_columnTags[column]->keys(), column);
+    }
 }
 
 void CollectionList::slotNewItems(const KFileItemList &items)
 {
     QStringList files;
 
-    for(KFileItemList::ConstIterator it = items.constBegin(); it != items.constEnd(); ++it)
-        files.append((*it).url().path());
+    for(const KFileItem &file : items) {
+        files.append(file.url().path());
+    }
 
     addFiles(files);
     update();
@@ -252,18 +259,17 @@ void CollectionList::saveItemsToCache() const
     s.setVersion(QDataStream::Qt_4_3);
 
     { // locked scope
-        QHash<QString, CollectionListItem *>::const_iterator it;
         QWriteLocker lock(&m_itemsDictLock);
 
-        for(it = m_itemsDict.begin(); it != m_itemsDict.end(); ++it) {
-            s << it.key();
-            s << (*it)->file();
+        for(const auto &[key, v] : as_const(m_itemsDict).asKeyValueRange()) {
+            s << key;
+            s << v->file();
         }
     }
 
     QDataStream fs(&f);
 
-    qint32 checksum = qChecksum(data.data(), data.size());
+    qint32 checksum = qChecksum(data);
 
     fs << qint32(Cache::playlistItemsCacheVersion)
        << checksum
@@ -300,7 +306,7 @@ void CollectionList::slotCheckCache()
     { // locked scope
         QWriteLocker lock(&m_itemsDictLock);
 
-        for(auto item : qAsConst(m_itemsDict)) {
+        for(auto item : std::as_const(m_itemsDict)) {
             if(!item->checkCurrent())
                 invalidItems.append(item);
         }
@@ -315,6 +321,7 @@ void CollectionList::slotRemoveItem(const QString &file)
 {
     QWriteLocker lock(&m_itemsDictLock);
     delete m_itemsDict[file];
+    m_itemsDict.remove(file);
 }
 
 void CollectionList::slotRefreshItem(const QString &file)
@@ -336,9 +343,9 @@ CollectionList::CollectionList(PlaylistCollection *collection) :
     spaction->setText(i18n("Show Playing"));
     connect(spaction, SIGNAL(triggered(bool)), SLOT(slotShowPlaying()));
 
-    connect(action<KToolBarPopupAction>("back")->menu(), SIGNAL(aboutToShow()),
+    connect(action<KToolBarPopupAction>("back")->popupMenu(), SIGNAL(aboutToShow()),
             this, SLOT(slotPopulateBackMenu()));
-    connect(action<KToolBarPopupAction>("back")->menu(), SIGNAL(triggered(QAction*)),
+    connect(action<KToolBarPopupAction>("back")->popupMenu(), SIGNAL(triggered(QAction*)),
             this, SLOT(slotPlayFromBackMenu(QAction*)));
     setSortingEnabled(false); // Temporarily disable sorting to add items faster.
 
@@ -513,9 +520,9 @@ void CollectionListItem::refresh()
         sharedData()->cachedWidths[i] = newWidth;
     }
 
-    for(PlaylistItemList::Iterator it = m_children.begin(); it != m_children.end(); ++it) {
-        (*it)->playlist()->update();
-        (*it)->playlist()->playlistItemsChanged();
+    for(PlaylistItem *item : m_children) {
+        item->playlist()->update();
+        item->playlist()->playlistItemsChanged();
     }
     if(treeWidget()->isVisible())
         treeWidget()->viewport()->update();
@@ -529,11 +536,12 @@ PlaylistItem *CollectionListItem::itemForPlaylist(const Playlist *playlist)
     if(playlist == CollectionList::instance())
         return this;
 
-    PlaylistItemList::ConstIterator it;
-    for(it = m_children.constBegin(); it != m_children.constEnd(); ++it)
-        if((*it)->playlist() == playlist)
-            return *it;
-    return 0;
+    for(PlaylistItem *item : as_const(m_children)) {
+        if(item->playlist() == playlist)
+            return item;
+    }
+
+    return nullptr;
 }
 
 void CollectionListItem::updateCollectionDict(const QString &oldPath, const QString &newPath)
@@ -549,10 +557,9 @@ void CollectionListItem::updateCollectionDict(const QString &oldPath, const QStr
 
 void CollectionListItem::repaint() const
 {
-    // FIXME repaint
-    /*QItemDelegate::repaint();
-    for(PlaylistItemList::ConstIterator it = m_children.constBegin(); it != m_children.constEnd(); ++it)
-        (*it)->repaint();*/
+    for(PlaylistItem *item : m_children) {
+        item->playlist()->update();
+    }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -581,8 +588,9 @@ CollectionListItem::~CollectionListItem()
 {
     m_shuttingDown = true;
 
-    foreach(PlaylistItem *item, m_children)
+    for(PlaylistItem *item : m_children) {
         delete item;
+    }
 
     CollectionList *l = CollectionList::instance();
     if(l) {
